@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.6.8+   2009/07/03
+ * Version: 1.6.9+   2011/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -172,12 +172,17 @@ int ccs_del_domain_acl(struct ccs_acl_info *acl)
 static int ccs_audit_execute_handler_log(struct ccs_execve_entry *ee,
 					 const bool is_default)
 {
+	int error;
 	struct ccs_request_info *r = &ee->r;
+	struct ccs_domain_info *domain = r->domain;
 	const char *handler = ee->handler->name;
+	r->domain = ccs_current_domain();
 	r->mode = ccs_check_flags(r->domain, CCS_MAC_FOR_FILE);
-	return ccs_write_audit_log(true, r, "%s %s\n",
-				   is_default ? KEYWORD_EXECUTE_HANDLER :
-				   KEYWORD_DENIED_EXECUTE_HANDLER, handler);
+	error = ccs_write_audit_log(true, r, "%s %s\n",
+				    is_default ? KEYWORD_EXECUTE_HANDLER :
+				    KEYWORD_DENIED_EXECUTE_HANDLER, handler);
+	r->domain = domain;
+	return error;
 }
 
 /**
@@ -771,14 +776,14 @@ struct ccs_domain_info *ccs_find_or_assign_new_domain(const char *domainname,
 			continue;
 		flag = false;
 		/***** CRITICAL SECTION START *****/
-		read_lock(&tasklist_lock);
+		ccs_tasklist_lock();
 		for_each_process(p) {
 			if (ccs_task_domain(p) != domain)
 				continue;
 			flag = true;
 			break;
 		}
-		read_unlock(&tasklist_lock);
+		ccs_tasklist_unlock();
 		/***** CRITICAL SECTION END *****/
 		if (flag)
 			continue;
@@ -1042,7 +1047,9 @@ static int ccs_check_environ(struct ccs_execve_entry *ee)
 {
 	struct ccs_request_info *r = &ee->r;
 	struct linux_binprm *bprm = ee->bprm;
-	char *arg_ptr = ee->tmp;
+	/* env_page->data is allocated by ccs_dump_page(). */
+	struct ccs_page_dump env_page = { };
+	char *arg_ptr; /* Size is CCS_EXEC_TMPSIZE bytes */
 	int arg_len = 0;
 	unsigned long pos = bprm->p;
 	int offset = pos % PAGE_SIZE;
@@ -1052,14 +1059,16 @@ static int ccs_check_environ(struct ccs_execve_entry *ee)
 	int error = -ENOMEM;
 	if (!r->mode || !envp_count)
 		return 0;
+	arg_ptr = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
+	if (!arg_ptr)
+		goto out;
 	while (error == -ENOMEM) {
-		if (!ccs_dump_page(bprm, pos, &ee->dump))
+		if (!ccs_dump_page(bprm, pos, &env_page))
 			goto out;
 		pos += PAGE_SIZE - offset;
 		/* Read. */
 		while (argv_count && offset < PAGE_SIZE) {
-			const char *kaddr = ee->dump.data;
-			if (!kaddr[offset++])
+			if (!env_page.data[offset++])
 				argv_count--;
 		}
 		if (argv_count) {
@@ -1067,8 +1076,7 @@ static int ccs_check_environ(struct ccs_execve_entry *ee)
 			continue;
 		}
 		while (offset < PAGE_SIZE) {
-			const char *kaddr = ee->dump.data;
-			const unsigned char c = kaddr[offset++];
+			const unsigned char c = env_page.data[offset++];
 			if (c && arg_len < CCS_MAX_PATHNAME_LEN - 10) {
 				if (c == '=') {
 					arg_ptr[arg_len++] = '\0';
@@ -1104,6 +1112,8 @@ static int ccs_check_environ(struct ccs_execve_entry *ee)
  out:
 	if (r->mode != 3)
 		error = 0;
+	kfree(env_page.data);
+	kfree(arg_ptr);
 	return error;
 }
 
@@ -1340,6 +1350,9 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 	struct task_struct *task = current;
 
 	/* Close the requested program's dentry. */
+	ee->obj.path1_dentry = NULL;
+	ee->obj.path1_vfsmnt = NULL;
+	ee->obj.validate_done = false;
 	allow_write_access(bprm->file);
 	fput(bprm->file);
 	bprm->file = NULL;
@@ -1460,6 +1473,8 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 		retval = PTR_ERR(filp);
 		goto out;
 	}
+	ee->obj.path1_dentry = filp->f_dentry;
+	ee->obj.path1_vfsmnt = filp->f_vfsmnt;
 	bprm->file = filp;
 	bprm->filename = ee->program_path;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
@@ -1557,6 +1572,9 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 3 && defined(CONFIG_MMU)
 	if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page, NULL) <= 0)
 		return false;
+#elif defined(AX_MAJOR) && AX_MAJOR == 3 && defined(AX_MINOR) && AX_MINOR >= 2 && defined(CONFIG_MMU)
+	if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page, NULL) <= 0)
+		return false;
 #else
 	page = bprm->page[pos / PAGE_SIZE];
 #endif
@@ -1577,6 +1595,8 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 	put_page(page);
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 3 && defined(CONFIG_MMU)
 	put_page(page);
+#elif defined(AX_MAJOR) && AX_MAJOR == 3 && defined(AX_MINOR) && AX_MINOR >= 2 && defined(CONFIG_MMU)
+	put_page(page);
 #endif
 	return true;
 }
@@ -1639,6 +1659,7 @@ int ccs_start_execve(struct linux_binprm *bprm)
  ok:
 	if (retval < 0)
 		goto out;
+	ee->r.profile = ee->r.domain->profile;
 	ee->r.mode = ccs_check_flags(ee->r.domain, CCS_MAC_FOR_ENV);
 	retval = ccs_check_environ(ee);
 	if (retval < 0)
@@ -1701,6 +1722,8 @@ int ccs_start_execve(struct linux_binprm *bprm)
 
 /**
  * ccs_finish_execve - Clean up execve() operation.
+ *
+ * @retval: Unused.
  */
 void ccs_finish_execve(int retval)
 {
